global g_helptxt;
global g_subMenuHiding;
global g_subMenuActive;
global _saved_criterion;
global _saved_water;

function findGenotypeAndName(name,genotype,pool)
{
for(var g in pool)
   if ((g.name==name) && (g.genotype==genotype))
       return g.index;
return -1;
}

function setInitialDataDictionary(g)
{
	g.data={"version":1,"evaluations":0,"selections":0,"edited":0,"imported":0,"time":0,"water_velocity":0.0};
}

function getInitialGenotype()
{
	return "RcqXXX(ddGGbbXX,DDggbbX)";
}

function addInitialGenotype()
{
	var g=GenePools[0].add(getInitialGenotype());
	g.name = "Initial genotype";
	setInitialDataDictionary(g);
}

function loadCurrent()
{
	GenePools[0].clear();
	Simulator.import("scripts_output" + File.pathseparator + "mobi-current.gen");
	if (GenePools[0].size == 0)
		addInitialGenotype();
}

function saveCurrent()
{
	Simulator.export("mobi-current.gen", 2, 0, 0);
}

function loadHistory()
{
	var pool=GenePools[0];
	pool.clear();
	addInitialGenotype();
	Simulator.import("scripts_output" + File.pathseparator + "mobi-history.gen");
	if (pool.size > 1)
		if (pool[0].genotype == pool[1].genotype)
			pool[1].delete();
  
	for(var i=0;i<pool.size;i++)
		{
		var g=pool[i];
		if (g.data->version!=1)
			{
			pool[i].delete();
			i--;
			}
		}
}

function saveHistory()
{
	Simulator.export("mobi-history.gen", 2, 0, 0);
}

function addHistory(g)
{
	loadHistory();
	
	// history limit: since quests are saved independently, we can remove old genotypes to prevent unlimited growth of history
	if (GenePools[0].size > 49)
		GenePools[0][0].delete();

	g.moveTo(GenePools[0]);
	g.data->time = Math.time;
	saveHistory();
}

const BREADCRUMB_YOUR_CREATURES="Your creatures";
const BREADCRUMB_CURRENT="Current";

function makeBreadcrumbPath(nodes)
{
var txt="";
for(var n in nodes)
  {
  if (txt!="") txt+=" > ";
  txt+=n;
  }
return txt;
}

function slideTopStatus()
{
GUI->topstatus.animSetBegin();
GUI->topstatus.setRect(0,-GUI.fontHeight(0),GLDisplay.size_x,GUI.fontHeight(0));
GUI->topstatus.animSetEnd();
GUI->topstatus.startAnimation(1);
}

function displayTopStatus(txt)
{
if (GUI->topstatus==null)
   {
   var lab=GUI.newLabel("topstatus",txt);
   lab.bgcolor=0xff000000; lab.align=0; lab.font=0;
   }
else
   GUI->topstatus.text=txt;
GUI->topstatus.setRect(0,0,GLDisplay.size_x,GUI.fontHeight(0));
GUI.setTimeout("topstatus",3,function slideTopStatus);
}

function ALsharedResetGUI()
{
	g_subMenuHiding = null;
	g_subMenuActive = null;
}

function addHelpButton(place)
{
	var but = GUI.newButton("helpbut", "");
	but.style = 1;
	but.image = "info-icon-shadow";
        but.addTo(place);
}

function handleHelp(elem)
{
	if (String.startsWith(elem.name, "help"))
	{
		if (GUI->help)
		{
			GUI.removeElement("help");
		}
		else
		{
			var h = GUI.newBubble("help", g_helptxt);
			h.addTo(GUI->helpbut);
			h.setAutolayout(0, 0, 0, 0, 0, 0, 1, 1);
		}
		return 1;
	}
	return 0;
}

function subMenuHidingFinished()
{
	for(var sub in g_subMenuHiding)
		sub.visible = 0;
	g_subMenuHiding = null;
	GUI.setTimeout("subhide", 0, null);
}

function handleSubMenu(elem, menu, dir, w, h)
{
	if (g_subMenuHiding)
		subMenuHidingFinished();
	var show = !elem.selected;
	elem.selected = show;
	var t = 0.2;
	if (show)
	{
	  	g_subMenuActive = [elem, menu, dir, w, h];
		t += 0.1 * menu.size;
		var begin = elem.absoluteRect;
		var end = elem.absoluteRect;
		end.width = w * GLDisplay.size_x;
		end.height = h * GLDisplay.size_y;
		if (end.x + end.width > GLDisplay.size_x) end.x = float(GLDisplay.size_x) - end.width;
		var dy;
		if (dir == 1) end.y += end.height;
		end.height *= 0.8;
		if (dir == 1)
		{
			dy = end.height;
			end.y -= dy;
		}
		if (dir == 0) dy = -end.height;
		for(var sub in menu)
		{
			end.y += dy;
			sub.visible = 1;
			sub.rect = begin;
			sub.animSetBegin();
			sub.setAutolayout(0, 0, 0, 0, end.x / GLDisplay.size_x, end.y / GLDisplay.size_y, end.width / GLDisplay.size_x, end.height / GLDisplay.size_y);
			sub.rect = end;
			sub.animSetEnd();
			sub.startAnimation(t);
			t -= 0.1;
		}
	}
	else
	{
	  	g_subMenuActive = null;
		for(var sub in menu)
		{
			sub.animSetBegin();
			sub.rect = elem.absoluteRect;
			sub.animSetEnd();
			sub.startAnimation(t);
			t += 0.2;
		}
		g_subMenuHiding = menu;
		GUI.setTimeout("subhide", t / 2, function subMenuHidingFinished);
	}
}

function closePopup()
{
if (g_subMenuActive)
  {handleSubMenu(g_subMenuActive[0],g_subMenuActive[1],g_subMenuActive[2],g_subMenuActive[3],g_subMenuActive[4]); return 1;}
if (GUI->help)
  {handleHelp(GUI->help); return 1;}
return 0;
}
